@@ -48,7 +48,7 @@ static char * kanamap[128]=
 	"gu", "ke", "ge", "ko", "go", "sa", "za", "shi",
 	"zhi", "su", "zu", "se", "ze", "so","zo", "ta",
 
-	"da", "chi","ji", "",   "tsu","zu", "te", "de",
+	"da", "chi","ji", "",   "tsu","du", "te", "de",
 	"to", "do", "na", "ni", "nu", "ne", "no", "ha",
 	
 	"ba", "pa", "hi", "bi", "pi", "fu", "bu", "pu",
@@ -92,9 +92,7 @@ void kanatoromaji(XChar2b *kstring, char *estring)
 
 	eptr[0]='\0';
 	
-	do
-	{
-
+	do{
 		if(strlen(estring)> (MAXROMAJI-3) ) {
 			puts("WARNING: MAXROMAJI overflow");
 			estring[MAXROMAJI-2]='X';
@@ -103,16 +101,14 @@ void kanatoromaji(XChar2b *kstring, char *estring)
 		}
 
 		/* how boring.. punctuation */
-		if(kptr->byte1 == 0x21)
-		{
+		if(kptr->byte1 == 0x21)	{
 			int val=kptr->byte2;
 			/* print character before this punc */
 			strcat(eptr, kanamap[lastchar]);
 
 			lastchar=0;
 			kptr++;
-			switch(val)
-			{
+			switch(val) {
 			   case 0x21:
 				strcat(eptr," ");
 				continue;
@@ -147,14 +143,12 @@ void kanatoromaji(XChar2b *kstring, char *estring)
 
 
 		/* This is a wierd parsing routine.. Try to ignore it!*/
-		if(kptr->byte2>128)
-		{
+		if(kptr->byte2>128){
 			strcat(eptr, "[bad byte2 found]");
 			kptr++;
 			continue;
 		}
-		switch(kptr->byte2)
-		{
+		switch(kptr->byte2){
 		   case 0x63:
 		   case 0x65:
 		   case 0x67:
@@ -181,8 +175,7 @@ void kanatoromaji(XChar2b *kstring, char *estring)
 		/* Put the appropriate prefix */
 		/* Later, will append "a", "u", or "o" */
 
-		switch(lastchar)
-		{
+		switch(lastchar){
 		   case 0x2d:	/*ki*/
 			strcat(eptr,"ky");
 			break;
@@ -223,8 +216,7 @@ void kanatoromaji(XChar2b *kstring, char *estring)
 			strcat(eptr, "_y");
 			break;
 		}
-		switch(kptr->byte2)
-		{
+		switch(kptr->byte2){
 		   case 0x63:
 			strcat(eptr, "a");
 			break;
@@ -245,17 +237,20 @@ void kanatoromaji(XChar2b *kstring, char *estring)
 }
 
 
-/* romajimap1 has to match romajimap2!! */
-/* Note that this is checked in order!!*/
+#define ROMAJIMAPSIZE 150 /* there is some empty space for growth */
+
+/* romajimap1 has to match romajimap2, in exact order!! */
 /* If we were nice, this would also be XChar2b encoded, but..*/
-static char *romajimap1[]=
+static char *romajimap1[ROMAJIMAPSIZE]=
 {
 	"shya", "shyu", "shyo",
 	"chya","chyu","chyo",
 	"sya","syu","syo",
 	"sha","shi", "shu","sho",
+	"jya","jyu","jyo",
 	"cha","chu","cho",
 	"gya","gyu","gyo",
+	"dzu","tzu",             /* same as "du" */
 
 	"hya","hyu","hyo",
 	"bya","byu","byo",
@@ -273,7 +268,7 @@ static char *romajimap1[]=
 	"ta","da","ti","di","tu","du","te","de","to","do",
 
 	"na","ni","nu","ne","no",
-	"n ",
+	"n ","nn",
 	"ha","ba","pa",
 	"hi","bi","pi",
 	"hu","fu","bu","pu",
@@ -289,15 +284,17 @@ static char *romajimap1[]=
 	"a","i","u","e","o",
 	
 	"-", "'","(",")", /* These are here not so much for romaji input,
-	                   * as to match dictionary search patterns
+	                   * as to match dictionary search patterns,
+			   * and translate kana to romaji.
 			   */
 	""
 	/* See also hardcodes for special keys in Handle_romajikana()
 	 * and matchromaji()
 	 */
 };
+
 /* must match order of romajimap1[]!!*/
-static XChar2b romajimap2[150][3]=
+static XChar2b romajimap2[ROMAJIMAPSIZE][3]=
 {
 	{{0x24, 0x37}, {0x24, 0x63}, {0, 0}},/*shya*/
 	{{0x24, 0x37}, {0x24, 0x65}, {0, 0}},/*shyu*/
@@ -317,6 +314,10 @@ static XChar2b romajimap2[150][3]=
 	/*{{0x24, 0x37}, {0x24, 0x67}, {0, 0}}, ??she??*/
 	{{0x24, 0x37}, {0x24, 0x67}, {0, 0}},/*sho*/
 
+	{{0x24, 0x38}, {0x24, 0x63}, {0, 0}},/*jya*/
+	{{0x24, 0x38}, {0x24, 0x65}, {0, 0}},/*jyu*/
+	{{0x24, 0x38}, {0x24, 0x67}, {0, 0}},/*jyo*/
+
 	{{0x24, 0x41}, {0x24, 0x63}, {0, 0}},/*cha*/
 	{{0x24, 0x41}, {0x24, 0x65}, {0, 0}},/*chu*/
 	{{0x24, 0x41}, {0x24, 0x67}, {0, 0}},/*cho*/
@@ -325,6 +326,9 @@ static XChar2b romajimap2[150][3]=
 	{{0x24, 0x2e}, {0x24, 0x65}, {0, 0}},/*gyu*/
 	{{0x24, 0x2e}, {0x24, 0x67}, {0, 0}},/*gyo*/
 
+	{{0x24, 0x45}, {0, 0}},/*dzu*/
+	{{0x24, 0x45}, {0, 0}},/*tzu*/
+
 	{{0x24, 0x52}, {0x24, 0x63}, {0, 0}},/*hya*/
 	{{0x24, 0x52}, {0x24, 0x65}, {0, 0}},/*hyu*/
 	{{0x24, 0x52}, {0x24, 0x67}, {0, 0}},/*hyo*/
@@ -403,6 +407,7 @@ static XChar2b romajimap2[150][3]=
 	{{0x24, 0x4e}, {0, 0}},/*no*/
 
 	{{0x24, 0x73}, {0, 0}},/*'n '*/
+	{{0x24, 0x73}, {0, 0}},/*'nn'*/
 
 	{{0x24, 0x4f}, {0, 0}},/*ha*/
 	{{0x24, 0x50}, {0, 0}},/*ba*/
@@ -456,54 +461,74 @@ static XChar2b romajimap2[150][3]=
 	{{0x24, 0x28}, {0, 0}},/*e*/
 	{{0x24, 0x2a}, {0, 0}},/*o*/
 
+	/* These are here really for kana-to-romaji only */
+	/* But make sure Handle_romajikana uses same values! */
 	{{0x21, 0x3c}, {0, 0}},/*- for elongation*/	
 	{{0x24, 0x43}, {0, 0}},/*"'" for small-tsu*/
-		/* These two must match ReadProdunciation() */
-	{{0x21, 0x4a}, {0, 0}},/*"(" for open-paren. */
-	{{0x21, 0x4b}, {0, 0}},/*")" for close-paren. */
+	{{0x21, 0x4a}, {0, 0}},/*"(" */
+	{{0x21, 0x4b}, {0, 0}},/*")" */
 
 	{{0}}
 	
 };
 
 
-/* suborutine for romajitokana()
- * return 1 if regular char* can be said to match kstring,
- * Like strstr, not like strcmp
+/* subroutine for romajitokana()
+ * romaji is one of the standard romaji tokens.
+ * kstring is user input.
+ *
+ * return 1 if kstring matches the known romaji combination.
  */
 static int matchromaji(char *romaji, XChar2b *kstring)
 {
-	while(*romaji!='\0')
-	{
-		switch(kstring->byte1){
-			case 0x21:
-			case 0x22:
-			case 0x23:
-				break;
-			default:
-				return 0;
+	while(*romaji!='\0'){
+		if(kstring->byte1==0){
+			return 0;
 		}
+
 		/*durn non-ascii punctuation codes...*/
+		/* IFF first char is JIS '_', then try to match
+		 * with "_ya" romaji combinations.
+		 * If we dont HAVE a _yXX combo to compare with,
+		 * then we dont have a match!
+		 */
 		if(kstring->byte1==0x21){
-			switch(kstring->byte2){
-				case 0x32: /* kana underscore */
-				if(*romaji != '_'){
+			if(kstring->byte2 == 0x32){
+				if(*romaji == '_'){
+					/* force match */
+					romaji++;
+					kstring++;
+					continue;
+				} else {
 					return 0;
 				}
 			}
-		} else {
-			if(*romaji != kstring->byte2)
+		}
+
+		if(*romaji != kstring->byte2){
 			return 0;
 		}
+
+		if(kstring->byte1!=0x23) {
+			/* Theoretically, this should be redundant. */
+			/* All matches should be ASCII to pseudo-ASCII now */
+
+			printf("WARNING: matchromaji hit nonascii:%x:%x\n",
+			       kstring->byte1,kstring->byte2);
+			return 0;
+		}
+
+
 		romaji++;
 		kstring++;
 	}
 
+	/* match found! */
 	return 1;
 }
-/* I'm throwing out the old romajitokana, and coding this more like
- * the jdrill routines. Now, we'll go from LEFT TO RIGHT, and
- * with special handling for 'n'
+/*
+ * We get called by process_kinput, on the "raw" string,
+ * before the string actually gets displayed.
  *
  * We assume kstart points to an array that we can modify in-place.
  * We also assume array is fully XChar2b encoded, as a "mix" of
@@ -511,36 +536,49 @@ static int matchromaji(char *romaji, XChar2b *kstring)
  * kana, and followed by 0 or more romaji. (encoded as 0x23,0x??)
  * We never should be more mixed than that. If we are, we don't
  * handle it well :-)
+ *
+ * Expected range of characters is determined by whatever
+ * Handle_romajikana puts in the string, minus whatever process_kinput
+ * has filtered out.
  */
 void romajitokana(XChar2b *kstart)
 {
 	XChar2b *kparse;
 	int matchcount;
 
-	/* start off by looking for pseudo-8-bit chars, aka romaji
-	 * Allow for JIS-symbol equivalent of ASCII punctuation, at
-	 * 0x21##, as well as pseudo-ascii at 0x23##
+	/* start off by looking for pseudo-8-bit chars, aka romaji.
+	 * look for first possible romaji string match, or end-of-string.
 	 */
-	while(kstart->byte1 > 0x23)
-	{
-		if(kstart->byte1 == 0)
-		{
-			/* NO non-kana CHARS! so just return */
-			return;
+	while(kstart->byte1!=0) {
+		if(kstart->byte1==0x23){
+			break;
+		}
+		/* special handling just for _ya combo */
+		if(kstart->byte1==0x21){
+			if(kstart->byte2==0x32){
+#ifdef DEBUG
+				puts("romajitokana: found __");
+#endif
+				break;
+			}
 		}
 		kstart++;
 	}
-
-	/* kstart now has first 8-bit char */
+	if(kstart->byte1==0){
+		/* no match! */
+		return;
+	}
+	
+	/* kstart now has first ascii-like 8-bit char
+	 * embedded in the XChar2b string
+	 */
 	kparse = kstart;
 
-	for(matchcount=0; romajimap1[matchcount][0]!='\0' ; matchcount++)
-	{
-		if(matchromaji(romajimap1[matchcount], kstart)==1)
-		{
+
+	for(matchcount=0; romajimap1[matchcount][0]!='\0' ; matchcount++){
+		if(matchromaji(romajimap1[matchcount], kstart)==1){
 			kparse=&romajimap2[matchcount][0];
-			while(kparse->byte1 != 0)
-			{
+			while(kparse->byte1 != 0){
 				kstart->byte1=kparse->byte1;
 				kstart->byte2=kparse->byte2;
 				kstart++;
@@ -552,12 +590,13 @@ void romajitokana(XChar2b *kstart)
 		}
 	}
 
-	/* we should really do the "special case for leading 'n'" now */
-
-	if((kstart[0].byte1==0x23) &&
-	   (kstart[0].byte2=='n'))
-		if((kstart[1].byte1!=0) &&
-		   (kstart[1].byte2!='y'))
+	/* Special case for 'n' in the middle of things.
+	 * Convert it ourselves to standalone 'n', unless potentially part
+	 * of "nya" or similar known compound
+	 */
+	if((kstart[0].byte1==0x23) && (kstart[0].byte2=='n'))
+		if((kstart[1].byte1!=0) &&  (kstart[1].byte2!='y') &&
+		   (kstart[1].byte2!='n'))
 		{
 			kstart[0].byte1=0x24;
 			kstart[0].byte2=0x73;
@@ -583,11 +622,11 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 	char *charpressed;
 	XChar2b addchar;
 	
-	if(e->type != KeyPress)
-	{
-		if(e->type == KeyRelease)
-		{
+	if(e->type != KeyPress){
+		if(e->type == KeyRelease){
+#ifdef DEBUG
 			puts("key released");
+#endif
 			return;
 		}
 		printf("Some other strange event found in for romaji: %d\n",
@@ -600,8 +639,7 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 	/* need XtGetActionkeysym to detect Shift-9=='(' */
 	inbetweenK = XtGetActionKeysym(e, NULL);
 
-	if(inbetweenK == (KeySym)NULL)
-	{
+	if(inbetweenK == (KeySym)NULL){
 		puts("NULL keysym on kana input???");
 		return;
 	}
@@ -612,8 +650,7 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 	 * Making byte1== 0x23, is a special signal to romajitokana,
 	 * that says "Hey, this is romaji"
 	 */
-	switch(inbetweenK)
-	{
+	switch(inbetweenK){
 	   case XK_BackSpace:
 	   case XK_Delete:
 		addchar.byte1 = 0x22;
@@ -621,7 +658,7 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 		process_kinput(addchar);
 		return;
 	   case XK_Return:
-	   /* pass our strange "accept" char*/
+	   /* pass our strange "accept" char, that means "do search now"*/
 		addchar.byte1 = paragraphglyph[0].byte1;
 		addchar.byte2 = paragraphglyph[0].byte2;
 		process_kinput(addchar);
@@ -633,65 +670,57 @@ void Handle_romajikana(Widget w, XtPointer closure, XEvent *e, Boolean *cont)
 		addchar.byte2 = ' ';
 		process_kinput(addchar);
 		return;
+	/* Special handling for punctuation, because JIS punctuation
+	 * conflicts with 8-bit numbers of ASCII punctuation
+	 */
 	   case XK_minus:
-		/* add "-", but in kana range
-		 * (for kanakana vowel extension )
-		 * Note that this "char" is invisible in JIS space.
-		 */
-		addchar.byte1 = 0x23;
-		addchar.byte2 = '-';	/* 0x2d */
-		process_kinput(addchar);
-		return;
-	   case XK_underscore:
-		/* nasty nasty hack that has to be matched in
-		 * matchromaji()
-		 */
 		addchar.byte1 = 0x21;
-		addchar.byte2 = 0x32; /* 0x32='_' in JIS */
+		addchar.byte2 = 0x3c;	/* -- in JIS */
 		process_kinput(addchar);
 		return;
 	   case XK_apostrophe:
-		/* hack to get "small tsu" to work.
-		 * Note that this "char" is invisible in JIS space
-		 */
-		addchar.byte1 = 0x23;
-		addchar.byte2 = '\'';	/* 0x3e*/
+		addchar.byte1 = 0x24;
+		addchar.byte2 = 0x43;  /* "'" in JIS */
 		process_kinput(addchar);
 		return;
 
 	   case XK_parenleft:
 		/*
 		 * Note that this "char" is invisible in JIS space
-		 * It is pseudo-ASCII
+		 * It is pseudo-ASCII.  See note on XK_minus
 		 */
-		addchar.byte1 = 0x23;
-		addchar.byte2 = '(';	/* 0x3e*/
+		addchar.byte1 = 0x21;
+		addchar.byte2 = 0x4a;
 			/* in JIS it would be 214a */
 		process_kinput(addchar);
 		return;
 	   case XK_parenright:
 		/*
-		 * Note that this "char" is invisible in JIS space
-		 * It is pseudo-ASCII
+		 * directly convert to JIS ourselves.
+		 * These must match whatever ReadPronunciation()
+		 * converts '()' in the dictionary to.
 		 */
-		addchar.byte1 = 0x23;
-		addchar.byte2 = ')';	/* 0x3e*/
+		addchar.byte1 = 0x21;
+		addchar.byte2 = 0x4b;
 			/* in JIS it would be 214b */
 		process_kinput(addchar);
 		return;
 		
+	   case XK_underscore:
+		addchar.byte1 = 0x21;
+		addchar.byte2 = 0x32; /* 0x2132 == '_' in JIS */
+		process_kinput(addchar);
+		return;
 	}
 	charpressed = XKeysymToString(inbetweenK);
 	if(charpressed == NULL)
 		return;
 
-
 #ifdef DEBUG
 	printf("got string \"%s\"\n", charpressed);
 #endif		
 	/* now use process_kinput, 222b is erase */
-	if((*charpressed <0x61) || (*charpressed >0x7a))
-	{
+	if((*charpressed <0x61) || (*charpressed >0x7a)){
 		/* outside range of ascii chars we like */
 		if((*charpressed!='(') && *charpressed != ')'){
 #ifdef DEBUG
