@@ -115,6 +115,7 @@ void
 handle_delete(Widget w,XtPointer closure,XEvent *event,Boolean *cont)
 {
 	XClientMessageEvent *cevent = (XClientMessageEvent *) event;
+	puts("DEBUG: handle_delete called !!\n");
 
 	if(cevent->type != ClientMessage) return;
 	if(cevent->message_type != wm_message) return;
@@ -135,15 +136,43 @@ handle_delete(Widget w,XtPointer closure,XEvent *event,Boolean *cont)
 	if(XtIsTransientShell(w)){
 		puts("   IS a 'transient' shell");
 	}
+
+
+	if(XtIsTopLevelShell(XtParent(w))){
+		puts("   PARENT is a top level shell");
+	}
+	if(XtIsShell(XtParent(w))){
+		puts("   PARENT is a  shell");
+	}
+	if(XtIsTransientShell(XtParent(w))){
+		puts("   PARENT is a 'transient' shell");
+	}
+
+	
 #endif /* DEBUG */
 
-	if(XtIsTransientShell(w)){
-		XtPopdown(w);
-	} else {
+
+	/* Problem here: there are three types of windows we have
+	   to deal with:
+	   "Option window" type  (shell, transient)
+	   multi-match window  (toplevel, shell)
+	   seach window (toplevel, shell: but DIFFERENT from multi somehow)
+	   XtPopdown works for the first two, but only UnmapWidget works
+	   for last kind.
+	   The only difference I can see, is that the search window
+	   has "toplevel" as its parent, which is not strictly
+	   speaking just a window. It is what XtVaAppInitialize() returns.
+	   It CONTAINS a window, but is not merely a window.
+	   searchwindow is made that way, to remain open, when
+	   the main window is iconified.
+	   I *could* just special-case it. But i hate special cases.
+	*/
+	XtPopdown(w);
+	if(isMapped(w)){
+		/* if it is still showing.. try harder! */
 		XtUnmapWidget(w);
 	}
 
-	/* NOTREACHED */
 }
 
 /* initgc:
