@@ -113,8 +113,7 @@ void SetXtrmString(char *string, char *value)
 	char namebuff[100];
 	char *pstring=namebuff;
 	XrmDatabase db = XrmGetDatabase(display);
-	if(db==NULL)
-	{
+	if(db==NULL){
 		perror("could not get x resource db?");
 		return;
 	}
@@ -154,6 +153,11 @@ void SetXtrmBoolean(char *string, Boolean value)
 	SetXtrmString(string, svalue);
 }
 
+#ifndef _POSIX_PATH_MAX
+#define _POSIX_PATH_MAX 1024
+#endif
+
+
 /*
  * Save our options, in Xrm format, in the user's home directory.
  * We can only save stuff if values have been set with the SetXtrmXXXX()
@@ -169,13 +173,12 @@ void SaveConfig()
 		"noBell",
 		"nousefile",
 		"usefile",
-		"dictlocation",
-		"edictlocation",
 		"kanjifont",
 		"smallkanji",
 		"englishfont",
 		"kdictfile",
 		"edictfile",
+		"radkfile",
 		"gradelevel",
 		"guessmode",
 		"questionmode",
@@ -190,16 +193,14 @@ void SaveConfig()
 
 	sprintf(fname,"%s/.kdrill", homedir);
 	preffile=fopen(fname,"w");
-	if(preffile==NULL)
-	{
+	if(preffile==NULL){
 		perror("could not open pref file .kdrill");
 		return;
 	}
 	fprintf(preffile, "! Do not hand-edit this file\n");
 	fprintf(preffile, "! It will get overwritten when you quit kdrill\n");
 	
-	while(*rname!=NULL)
-	{
+	while(*rname!=NULL){
 		char classbuff[100], infobuff[100];
 		sprintf(classbuff,*rname);
 
