@@ -96,6 +96,8 @@ void StripBrackets(char *dest,unsigned char *source)
 				break;
 			case '}':
 				break;
+			case '\n':
+				*dest++='\0';
 			default:
 				*dest++ = *parse;				
 		}
@@ -154,6 +156,15 @@ int StripSlash(char *topdest,unsigned char *source)
 	return 0;
 }
 
+/* Given a translation, return the index into translations[] that it
+ * sits at
+ */
+int trans_to_index(TRANSLATION trans)
+{
+	return trans->kdrill_index;
+}
+
+
 /* read in kanji/kana part of edictfile line.
  * format is:
  *
@@ -438,6 +449,8 @@ void readedict()
 		return;
 	}
 
+	edictname=malloc(strlen(edict)+1);
+	strcpy(edictname, edict);
 	fp = open_compressed(edictname);
 	if(fp == NULL)
 	{
@@ -514,7 +527,7 @@ void readedict()
 		}
 		
 		if(StripSlash(newk->english, parse)!=0){
-			fprintf(stderr, "bad line: 5%s\n", instring);
+			fprintf(stderr, "bad line: %s\n", instring);
 		}
 
 		/* Success! Set pointers appropriately */
@@ -542,8 +555,7 @@ void readedict()
 	puts("");
 
 	puts("NOTE: an \"infinity\" sign means there is no kanji.");
-	puts("Switch to \"show meaning\" in the options window, for a more");
-	puts("interesting display");
+	puts("  Switch to \"show meaning\" option to show alternates.");
 
 	return;
 
@@ -557,7 +569,7 @@ short skipfromthree(int one, int two, int three){
       if((one>0xf) | (two>0xf) | (three>0xff) | (SKIPnum <0) )
       {
 #ifdef DEBUG
-              printf("corrupted SKIP ("Px-x-x") entry: %d-%d-%d\n",
+              printf("corrupted SKIP ('Px-x-x') entry: %d-%d-%d\n",
                      one, two, three);
 #endif
               return 0;
@@ -592,7 +604,7 @@ short parseskip(char *input)
 	if(*input != '-')
 	{
 #ifdef DEBUG
-		puts("corrupted SKIP ("Px-x-x") entry");
+		puts("corrupted SKIP ('Px-x-x') entry");
 #endif		
 		return 0;
 	}
@@ -605,7 +617,7 @@ short parseskip(char *input)
 	if(*input != '-')
 	{
 #ifdef DEBUG
-		puts("corrupted SKIP ("Px-x-x") entry");
+		puts("corrupted SKIP ('Px-x-x') entry");
 #endif		
 		return 0;
 	}
@@ -638,6 +650,8 @@ void readstructs(){
 		fprintf(stderr,"kdictfile set to 'none'. Skippping.\n");
 		return;
 	}
+	dictname=malloc(strlen(dict)+1);
+	strcpy(dictname, dict);
 	fp = open_compressed(dictname);
 	if(fp == NULL)
 	{
@@ -646,7 +660,7 @@ void readstructs(){
 		return;
 	}
 	
-	printf("opened dictionary %s \n",dictname);
+	printf("Opened dictionary %s \n",dictname);
 
 
 	if(fp ==NULL){
