@@ -9,6 +9,10 @@
 #include <StringDefs.h>
 #include <Xfuncs.h>
 
+#include <Xatom.h>
+#include <Xmu/Atoms.h>
+#include <Xmu/StdSel.h>
+
 #include <Shell.h>
 
 #include <Xaw/Command.h>
@@ -16,8 +20,13 @@
 #include <Xaw/Form.h>
 #include <Xaw/Box.h>
 #include <Xaw/AsciiText.h>
+#include <Xaw/Paned.h>
 #include <cursorfont.h>
 
+/* for _XawTextGetSTRING(), except actually including it gets messy  */
+/*#include <Xaw/TextP.h>*/
+
+
 #include "defs.h"
 #include "externs.h"
 #include "game.h"
@@ -26,13 +35,20 @@
 #include "convert.h"
 #include "multikanji.h"
 #include "strokesearch.h"
-
+#include "radsearch.h"
+#include "readfile.h"
 #include "searchwidgets.h"
-
+#include "learn.h"
 #include "utils.h"
 #include "init.h"
 #include "log.h"
 
+/* I dont like doing this. But there doesnt seem to be a reliable
+ * way to tell a widget "be the entire width of the form" that
+ * I have found
+ */
+#define SEARCHWIDTH 450
+
 
 Cursor textcursor;
 
@@ -51,7 +67,7 @@ Widget searchkanjiW,searchokuW; /* display current kanji here */
 XChar2b kanastring[MAXKANALENGTH]={{0,0}};
 
 /* flag, if we only match kanji/kana at beginning of translation */
-int match_onlyatstart=0;
+int match_onlyatstart=0, match_onlyactive=0;
 
 
 static char *searchAccel = 
@@ -59,8 +75,6 @@ static char *searchAccel =
 
 
 
-
-
 /* This gets called after clicking on the "search" button, or
  *  using the keyboard accelerator. It pops up the search window.
  */
@@ -103,18 +117,20 @@ void SearchCallback(Widget w,XtPointer client_data, XtPointer call_data)
 }
 
 /* handle toggle for "match only if kana STARTS a phrase" */
-/* actual search curerntly done in matchkana() */
-void Handle_starttoggle(Widget w,XtPointer client_data, XtPointer call_data)
+void ToggleMatchstart(Widget w,XtPointer client_data, XtPointer call_data)
 {
-	Pixel fg,bg;
 	match_onlyatstart=!match_onlyatstart;
 
-	XtVaGetValues(w, XtNforeground, &fg,
-		      XtNbackground, &bg, NULL);
+	ReverseButton(w);
 
-	XtVaSetValues(w, XtNforeground, bg,
-		      XtNbackground, fg, NULL);
-		    
+}
+
+/* handle toggle for "match only active kanji" */
+void ToggleActivesearch(Widget w,XtPointer client_data, XtPointer call_data)
+{
+	match_onlyactive=!match_onlyactive;
+
+	ReverseButton(w);
 }
 
 /* deal with matchkanji button. */
@@ -124,25 +140,28 @@ void Handle_matchkanji(Widget w,XtPointer client_data, XtPointer call_data)
 	findkanjiall(lastsearch->kanji);
 }
 
-/* Given a translation, return the index into translations[] that it
- * sits at
- */
-int trans_to_index(TRANSLATION trans)
+/* Handle toggle callback for "[usefile]" toggle button */
+/* Parallel func to multiUcallback() */
+static void 
+ToggleUsefile(Widget w,XtPointer client_data, XtPointer call_data)
 {
-	return trans->kdrill_index;
-}
-
-/* handle toggle for "This kanji in in usefile" button*/
-void Handle_usetoggle(Widget w,XtPointer client_data, XtPointer call_data)
-{
-	Pixel fg,bg;
 	int kcount=trans_to_index(lastsearch);
 	SetUseKanji(kcount, !InUsefile(kcount));
+
+	if(InUsefile(kcount)){
+		HighlightButton(w);
+	} else {
+		UnhighlightButton(w);
+	}
+
+	DisplayLearnChar(); /* on the off-chance the Learn window shows same*/
+	RefreshMultiLabels();
+
 	CountKanji();
 }
 
-/* handle "Show" usefile button */
-void Handle_showusefile(Widget w,XtPointer client_data, XtPointer call_data)
+/* Callback for button to  "Show usefile" */
+void ShowUsefile(Widget w,XtPointer client_data, XtPointer call_data)
 {
 	setstatus("Showing usefile...");
 
@@ -151,10 +170,11 @@ void Handle_showusefile(Widget w,XtPointer client_data, XtPointer call_data)
 
 /* Update the labels for the info widgets at the top of search window.*/
 /* Assumes variable "lastsearch" points to currently desired TRANSLATION */
-void updatesearchlabels()
+void UpdateSearchlabels()
 {
 	TRANSLATION This=lastsearch;
 	int kanjinum=0, kindex;
+	XChar2b radlist[MAXRADICALS+1];
 
 	if(search_popup==NULL)
 		return;
@@ -173,11 +193,9 @@ void updatesearchlabels()
 
 	kindex=trans_to_index(This);
 
-	if(InUsefile(kindex))
-	{
+	if(InUsefile(kindex)) {
 		HighlightButton(searchnumbers[INUSEFILE]);
-	} else
-	{
+	} else {
 		UnhighlightButton(searchnumbers[INUSEFILE]);
 	}
 
@@ -186,23 +204,21 @@ void updatesearchlabels()
 	 *  ('#x' window)
 	 */
 
-	if(This->kanji != NULL)
-	  if(This->kanji[1].byte1 == 0)
-	  {
-		kanjinum = This->kanji[0].byte1;
-		kanjinum = kanjinum <<8;
-		kanjinum |= This->kanji[0].byte2;
-		
-	  }
-	if(kanjinum == NOKANJI)
-	{
+	if(This->kanji != NULL){
+		if(This->kanji[1].byte1 == 0)
+		{
+			kanjinum = This->kanji[0].byte1;
+			kanjinum = kanjinum <<8;
+			kanjinum |= This->kanji[0].byte2;
+		}
+	}
+	if(kanjinum == NOKANJI) {
 		kanjinum = 0;
 	}
 
 	SetWidgetHexval(searchnumbers[POUND_INPUT], kanjinum);
 
-	if(romajiswitch==1)
-	{
+	if(romajiswitch==1) {
 		char stringbuff[MAXROMAJI+1];
 		/*translate all kana into romaji */
 		/* translate to buffer, then set widget string*/
@@ -215,8 +231,7 @@ void updatesearchlabels()
 		      XtNlabel,stringbuff,
 		      NULL);
 	
-	} else
-	{
+	} else {
 		XtVaSetValues(searchwidgets[KANA_W],
 		      XtNencoding, XawTextEncodingChar2b,
 		      XtNfont,smallkfont,
@@ -226,6 +241,17 @@ void updatesearchlabels()
 	}
 	XtVaSetValues(searchwidgets[ENGLISH_W],
 		      XtNlabel,This->english,NULL);
+
+	XtVaSetValues(searchnumbers[RADLIST_D],XtNlabel,"\0\0",NULL);
+	if(kanjinum!=0){
+		radlist[0].byte1=0;
+		/* a single kanji, should have radicals associated with it*/
+		FindRadicals(This->kanji[0],radlist);
+		if(radlist[0].byte1 != 0){
+			XtVaSetValues(searchnumbers[RADLIST_D],
+				XtNlabel,radlist,NULL);
+		}
+	}
 	
 }
 
@@ -266,7 +292,7 @@ void printsearch(TRANSLATION This)
 		return;
 	}
 	lastsearch = This;
-	updatesearchlabels();
+	UpdateSearchlabels();
 }
 
 /* SearchRefresh()
@@ -274,7 +300,7 @@ void printsearch(TRANSLATION This)
  */
 void SearchRefresh()
 {
-	updatesearchlabels();
+	UpdateSearchlabels();
 	if(isMapped(search_popup))
 		printsearch(lastsearch);
 }
@@ -288,156 +314,253 @@ long TextLength (w)
                               XawstAll, XawsdRight, 1, TRUE);
 }
 
-/* This tries using the secretnasty stuff in xclipboard
- * but fails.
- * We don't do anything in this routine any more.
- * but its nice to keep around for debug purposes.
- */
-void doFunkyGrab(Widget textwin)
-{
-	int length; /* this is nastynastynasty...*/
-	XPointer value;
-	int strparse=0;
-	char *str;
+/* pass in a string like "\x{1234}" and get back a single int val */
+static int unescape_kanji(char *string){
+	int kchar[4];
+	int count, newval;
 
-	length=TextLength(textwin);
-
-	/* Okay, this is pseudo-undocumented. sorry. its what works! */
-	value = (XPointer)_XawTextGetSTRING((TextWidget) textwin, 0, length);
-	if(value==NULL)
-	{
-		printf("value is a dud");
-		return;
+	if(strlen(string)<8){
+		fprintf(stderr,"unescape_kanji passed bad string\n");
+		return 0;
 	}
-	str=value;
-	printf("Using XawTextGetSTRING, we get:\n  ");
-	for(strparse=0; strparse<length; strparse++)
-	{
-		printf("%.2x ",str[strparse]);
-
+	if(string[0]!='\\'){
+		fprintf(stderr,"unescape_kanji passed bad string\n");
+		return 0;
 	}
-	puts("");
-	
+
+	/* I dont know of a ubiquitous "hex to int" function like
+	 * atoi
+	 */
+	for(count=0; count<4; count++){
+		if(('A' <= string[count+3]) && (string[count+3] <='F')){
+			kchar[count]=string[count+3] - 'A' + 10;
+			
+		} else if(('a' <= string[count+3]) && (string[count+3] <='f')){
+			kchar[count]=string[count+3] - 'a' + 10;
+			
+		} else {
+			kchar[count]=string[count+3] - '0';
+		}
+	}
+
+	newval=(kchar[0]<<12) +(kchar[1]<<8) +(kchar[2]<<4) +kchar[3];
+
+	return newval;
 }
 
-/*
- * Kay, this is ugly.
- * This whole routine is because I'm too lazy to jump through the
- * incredible hoops of learning how to do X cut-n-paste properly.
- * So, I made a fake drop-target out of a asciiTextWidget, and
- * made it so we also get notified of a mouseclick.
- * Then, we have to do nasty decoding to JIS, and THEN
- * do a lookup ;-)
+/* take various formats of JIS, etc,
+ * [from cut-n-paste]
+ * convert to what we expect, and call
+ * findkanjiall() on resulting string, IF we recognize something.
  *
- * We do this on button RELEASE, so that we can be reasonably sure
- * the widget has had the text inserted already.
+ * We Assume buffer is AT LEAST 2 bytes long !!!
+ *
+ * POSSIBILITIES:
+ * 1. "raw" JIS, that looks to us like
+ *   0xffffff##, 0xffffff##
+ *
+ * 2. prefaced SJIS, that looks to us like
+ *   0x1b, 0x2d, 0x41, 0xffffff##, 0xffffff##
+ *
+ * 2.1 prefaced JIS, that looks to us like
+ *   0x1b, 0x2d, 0x41, 0x####, 0x####, 0x1b, 0x28
+ * 
+ * 3. A different (older?) kind of prefaced JIS,that looks like
+ *   0x1b, 0x24, 0x29, 0x42, blahblahblah
+ * 
+ * 4. stripped JIS (from rxvt), that is going to be tricky to tell.
+ *
+ * return 1 on recognizeable JIS string, or 0 otherwise.
  */
-void Handle_searchPasted(Widget widget, XtPointer cl, XEvent *e, Boolean *cont)
-{
-	/* only handle up to three chars */
+static int doJISConvert(char *inputstring){
+	/* only handle up to three encoded chars. Ignore more. */
 	XChar2b kstring[4];
-	String str;
-	int strparse;
-	
-	if(e->type != ButtonRelease)
-		return;
+	char *strstart=NULL;
+	int kstr_ndx;
 
-	bzero(kstring, sizeof(kstring));
+	bzero(kstring, sizeof(XChar2b) * 4);
 
-/* alternate way of getting pasted value */
-/*	doFunkyGrab(widget);*/
+	/* SPECIAL CASE, with EARLY RETURN !! */
+	/* We have an ascii-hex-encoded string here */
+	/* and... It is probably actually UNICODE */
+	if(strncmp("\\x{",inputstring,3) == 0){
+		int stlen=strlen(inputstring);
+		/* pan newsreader actually uses its own funky encoding */
+		int tmpval=unescape_kanji(inputstring);
 
-	XtVaGetValues(widget,XtNstring,&str,NULL);
+		if(tmpval>= 0){
+			kstring[0].byte1=(tmpval&0xff00)>>8;
+			kstring[0].byte2=(tmpval&0xff);
 
-	strparse=0;
+		}
+		if(stlen>=16){
+			tmpval=unescape_kanji(&inputstring[8]);
+			if(tmpval>= 0){
+				kstring[1].byte1=(tmpval&0xff00)>>8;
+				kstring[1].byte2=(tmpval&0xff);
+			}
+		}
+		if(stlen>=24){
+			tmpval=unescape_kanji(&inputstring[16]);
+			if(tmpval>= 0){
+				kstring[2].byte1=(tmpval&0xff00)>>8;
+				kstring[2].byte2=(tmpval&0xff);
+			}
+		}
+		findunicodestring(kstring);
 
-	if(str[strparse]=='\0'){
-		doFunkyGrab(widget);
+		return 1;
+		
 	}
-#define OLDDEBUG
-#ifdef OLDDEBUG
-	puts("Handle_searchPasted: values of XtNstring pasted are:");
-	while(str[strparse] != '\0');
-	{
-		printf("%.2x.",str[strparse++]);
+
+	/* Otherwise, we have lightly encoded, or raw, JIS to parse */
+
+
+	if(inputstring[0]&0xffffff00){
+		strstart=&inputstring[0];
+	} else if(strncmp("\x1b\x24\x29\x42",inputstring,4) == 0){
+		strstart=&inputstring[4];
+	} else if(strncmp("\x1b\x2d\x41",inputstring,3) == 0){
+		strstart=&inputstring[3];
+	} else if(strncmp("\x1b\x2d\x42",inputstring,3) == 0){
+		/* We should never normally see this, because
+		 * the Xaw routines seem to autoconvert this to the
+		 * 0xffffff format instead
+		 */
+		strstart=&inputstring[3];
+	}
+
+
+	if((strstart==NULL) && (kstring[0].byte1==0)){
+		/* ugh. Unfortunately, rxvt hands us unencoded stuff,
+		 * which in theory might just be plain ascii.
+		 * Attempt to treat it as JIS, if first char hits
+		 * a known kanji
+		 */
+		int tmpndx=0;
+		tmpndx=(inputstring[0]<<8) |(inputstring[1]&0xff);
+		if((tmpndx>MAXKANJIALLOWED) ||(tmpndx< MINKANJIALLOWED)){
+			return 0;
+		}
+		if(translations[tmpndx] == NULL){
+			return 0;
+		}
+		strstart=&inputstring[0];
+	}
+
+	/* Okay, we have valid string. Convert up to 3 chars to JIS
+	 * format that we use internally, so we can search on it
+	 */
+	kstr_ndx=0;
+	while(kstr_ndx<3){
+		if((strstart[0]==0) || strstart[1]==0) {
+			break;
+		}
+		kstring[kstr_ndx].byte1=*strstart++ & 0x7f;
+		kstring[kstr_ndx].byte2=*strstart++ & 0x7f;
+		kstr_ndx++;
+	}
+	if(kstr_ndx==0) return 0;
+
+	kstring[kstr_ndx].byte1=0;
+	kstring[kstr_ndx].byte2=0;
+	findkanjiall(&kstring[0]);
+
+	return 1;
+
+}
+
+
+
+/* copy cut-n-paste buffer into local storage, and do search on it 
+ * if possible.
+ * Triggered by pasteCallback()
+ */
+void
+copybuffer(Widget w, XtPointer client_data, Atom *selection, Atom *type, 
+            XtPointer value, unsigned long *length, int *format)
+{
+	int buflen=(int)*length;
+	char *valuestring=(char*)value;
+
+	if(*type==0){
+#ifdef DEBUG
+		puts("DEBUG copybuffer has *type=0. returning.");
+#endif
+		return;
+	}
+
+
+	if(buflen==0){
+#ifdef DEBUG
+		puts("DEBUG: paste len==0. ignoring");
+#endif
+		return;
+	}
+	if(buflen<2){
+#ifdef DEBUG
+		puts("DEBUG: paste len==1. ignoring");
+#endif
+		XtFree(value);
+		return;
+	}
+#ifdef DEBUG
+	puts("copybuffer: values of XtNstring pasted are:");
+	while(bufparse <buflen) {
+		printf("%.2x.",valuestring[bufparse++]);
 
 	} 
 	puts("");
-	printf("strparse is now %d\n", strparse);
-#else
-	while(str[strparse++] != '\0');
+	printf("  total string len == %d\n", buflen);
 #endif
+	doJISConvert(valuestring);
 
-	if(strparse<2)
-	{
-		puts("Paste error: Less than 2 bytes pasted");
-		return;
-	}
 
-	/* POSSIBILITIES:
-	 * 1. "raw" JIS, that looks to us like
-	 *   0xffffff##, 0xffffff##
+	XtFree(value);
+}
+
+
+/* "Handler" for "<paste kanji>" button in search window.
+ * Completely obsoletes the old nasty Handle_searchPasted() hack I had before.
+ * This takes the high-level X clipboard stuff, and hands it over to
+ * copybuffer()
+ */
+void Handle_paste(Widget widget, XtPointer closure, XEvent *e, Boolean *cont)
+{
+	Time timestamp;
+	Atom clipboardatom;
+
+	timestamp=CurrentTime;
+
+	/*
+	puts("DEBUG: using XA_CLIPBOARD for cutnpaste");
+	clipboardatom=XA_CLIPBOARD(display);
+	*/
+
+	clipboardatom=XA_PRIMARY;
+
+	/* I originally used XA_STRING as the 'type', but
+	 * that did not work with kterm.
+	 * Once I changed to XA_TEXT(), it seems to work for both
+	 * kterm, AND netscape4.x
 	 *
-	 * 2. prefaced JIS, that looks to us like
-	 *   0x1b, 0x2d, 0x41, 0xffffff##, 0xffffff##
-	 * 
-	 * 3. A different (older?) kind of prefaced JIS,that looks like
-	 *   0x1b, 0x24, 0x29, 0x42, blahblahblah
+	 * call copybuffer() with the value.
 	 */
+	XtGetSelectionValue(widget, clipboardatom, XA_TEXT(display),
+			    copybuffer, NULL,
+			    timestamp);
+}
 
-	if(str[0]&0xffffff00)
-	{
-		puts("DEBUG: pure JIS paste");
 
-		/* just PURE JIS (probably) */
-		kstring[0].byte1=str[0]&0x7f;
-		kstring[0].byte2=str[1]&0x7f;
-
-		if(str[2]&0xffffff00)
-		{
-			kstring[1].byte1=str[2]&0x7f;
-			kstring[1].byte2=str[3]&0x7f;
-		}
-		if(str[4]&0xffffff00)
-		{
-			kstring[2].byte1=str[4]&0x7f;
-			kstring[2].byte2=str[5]&0x7f;
-		}
-
-	}
-	else if (str[0] == 0x1b){
-		int startchar=0;
-		/* do goodstuff.. figure out which of the two it is */
-		if((str[1]==0x2d) &&
-		   (str[2]==0x41)){
-			startchar=3;
-		}
-		else if((str[1]==0x24) &&
-			(str[2]==0x29) &&
-			(str[3]==0x42)){
-			startchar=4;
-		}else{
-			puts("DEBUG: unrecognized paste encoding");
-			return;
-		}
-		
-		kstring[0].byte1=str[startchar]&0x7f;
-		kstring[0].byte2=str[startchar+1]&0x7f;
-		if(str[startchar+2] != 0)
-		{
-			kstring[1].byte1=str[startchar+2]&0x7f;
-			kstring[1].byte2=str[startchar+3]&0x7f;
-			if(str[startchar+4] != 0)
-			{
-				kstring[2].byte1=str[startchar+4]&0x7f;
-				kstring[2].byte2=str[startchar+5]&0x7f;
-			}
-		}
-	}
-
-	XtVaSetValues(widget,XtNstring,"\0",NULL);
-
-	findkanjiall(&kstring[0]);
+/* This gets called when someone presses a mousebutton in the big kanji
+ * display string, in the search popup.
+ * Our job is to split up the string into multiple kanji, and
+ * popopulate the multikanji popup, so that the user can deconstruct
+ * the string into separate kanji
+ */
+void Handle_kanji_split(Widget widget, XtPointer closure, XEvent *e, Boolean *cont)
+{
+	split_kanji_search();
 }
 
 
@@ -449,66 +572,95 @@ void Handle_searchPasted(Widget widget, XtPointer cl, XEvent *e, Boolean *cont)
 void makenumbers(Widget parent){
 	searchnumbers[G_LABEL] = XtVaCreateWidget("0",labelWidgetClass,parent,
 		       XtNlabel,"G:",      XtNborderWidth,0,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[G_INPUT] = XtVaCreateWidget("1",asciiTextWidgetClass,parent,
+
+	/* G_INPUT is read-only, not really "INPUT".
+	 * but keep as asciiText so UpdateSearchLabels()
+	 * can have a uniform code layout.
+	 */
+	searchnumbers[G_INPUT] = XtVaCreateWidget("sn1",asciiTextWidgetClass,parent,
 		       XtNfromHoriz,searchnumbers[G_LABEL],
 		       XtNstring,"  ",  XtNdisplayCaret,False,
-		       XtNwidth,30,	/* leave this short */
+		       XtNwidth,30,	/* leave this short, match mainwin */
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[H_LABEL] = XtVaCreateWidget("6",labelWidgetClass,parent,
+	searchnumbers[H_LABEL] = XtVaCreateWidget("sn6",labelWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
 		       XtNlabel,"H:",   XtNborderWidth,0,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[H_INPUT] = XtVaCreateWidget("7",asciiTextWidgetClass,parent,
+	searchnumbers[H_INPUT] = XtVaCreateWidget("sn7",asciiTextWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
 		       XtNfromHoriz,searchnumbers[G_LABEL],
 		       XtNstring," ",  XtNwidth,INPUTWIDTH,
 		       XtNeditType,XawtextEdit,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
 
-	searchnumbers[F_LABEL] = XtVaCreateWidget("2",labelWidgetClass,parent,
+	searchnumbers[F_LABEL] = XtVaCreateWidget("sn2",labelWidgetClass,parent,
 		       XtNfromHoriz,searchnumbers[H_INPUT],
 		       XtNlabel,"F:",  XtNborderWidth,0,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[F_INPUT] = XtVaCreateWidget("3",asciiTextWidgetClass,parent,
+	searchnumbers[F_INPUT] = XtVaCreateWidget("sn3",asciiTextWidgetClass,parent,
 		       XtNfromHoriz,searchnumbers[F_LABEL],
 		       XtNstring,"",  XtNwidth,INPUTWIDTH,
 		       XtNeditType,XawtextEdit,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[N_LABEL] = XtVaCreateWidget("8",labelWidgetClass,parent,
+	searchnumbers[N_LABEL] = XtVaCreateWidget("sn8",labelWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
 		       XtNfromHoriz,searchnumbers[H_INPUT],
 		       XtNlabel,"N:",   XtNborderWidth,0,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[N_INPUT] = XtVaCreateWidget("9",asciiTextWidgetClass,parent,
+	searchnumbers[N_INPUT] = XtVaCreateWidget("sn9",asciiTextWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
 		       XtNfromHoriz,searchnumbers[F_LABEL],
 		       XtNstring," ",  XtNwidth,INPUTWIDTH,
 		       XtNeditType,XawtextEdit,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
 
-	searchnumbers[POUND_LABEL] = XtVaCreateWidget("4",labelWidgetClass,parent,
+	searchnumbers[POUND_LABEL] = XtVaCreateWidget("sn4",labelWidgetClass,parent,
 		       XtNfromHoriz,searchnumbers[F_INPUT],
 		       XtNfromHoriz,searchnumbers[N_INPUT],
 		       XtNlabel,"#x:",  XtNborderWidth,0,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[POUND_INPUT] = XtVaCreateWidget("5",asciiTextWidgetClass,parent,
+	searchnumbers[POUND_INPUT] = XtVaCreateWidget("sn5",asciiTextWidgetClass,parent,
 		       XtNfromHoriz,searchnumbers[POUND_LABEL],
 		       XtNstring,"",  XtNwidth,INPUTWIDTH,
 		       XtNeditType,XawtextEdit,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
-	searchnumbers[U_LABEL] = XtVaCreateWidget("10",labelWidgetClass,parent,
+	searchnumbers[U_LABEL] = XtVaCreateWidget("sn10",labelWidgetClass,parent,
 		       XtNfromVert,searchnumbers[G_LABEL],
 		       XtNfromHoriz,searchnumbers[F_INPUT],
 		       XtNfromHoriz,searchnumbers[N_INPUT],
 		       XtNlabel,"Ux:",   XtNborderWidth,0,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
 	searchnumbers[U_INPUT] = XtVaCreateWidget(
 			"11",asciiTextWidgetClass,parent,
 			XtNfromVert,searchnumbers[G_LABEL],
-			XtNfromHoriz,searchnumbers[U_LABEL],
+			XtNfromHoriz,searchnumbers[POUND_LABEL],
 			XtNstring," ",  XtNwidth,INPUTWIDTH,
 			XtNeditType,XawtextEdit,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 			NULL);
 
 	searchnumbers[INUSEFILE] = XtVaCreateWidget(
@@ -516,6 +668,8 @@ void makenumbers(Widget parent){
 			XtNlabel, "usefile",
 			XtNfromHoriz,searchnumbers[U_INPUT],
 			XtNhorizDistance, 20,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
 	searchnumbers[SHOWUSEFILE] = XtVaCreateWidget(
 			"12",commandWidgetClass,parent,
@@ -525,8 +679,31 @@ void makenumbers(Widget parent){
 			XtNfromHoriz,searchnumbers[U_INPUT],
 			XtNfromVert,searchnumbers[INUSEFILE],
 			XtNhorizDistance, 20,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
 		       NULL);
 
+	searchnumbers[RADLIST_L] = XtVaCreateManagedWidget("radlist_l",
+			labelWidgetClass,
+			parent,
+			XtNlabel,"Radicals:",
+			XtNborderWidth, 0,
+			XtNfromVert,searchnumbers[H_LABEL],
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
+			NULL);
+	searchnumbers[RADLIST_D] = XtVaCreateManagedWidget("radlist_d",
+			labelWidgetClass,
+			parent,
+			XtNfromVert,searchnumbers[H_LABEL],
+			XtNfromHoriz,searchnumbers[RADLIST_L],
+			XtNlabel,"                  ",
+			XtNencoding,XawTextEncodingChar2b,
+			XtNfont,smallkfont,
+			XtNright, XawChainRight,
+			XtNleft,  XawChainLeft,
+			NULL);
+
 	XtManageChildren(searchnumbers,NUM_OF_N);
 }
 
@@ -540,23 +717,37 @@ void maketopsearchstuff(Widget parent){
 	searchkanjiW = XtVaCreateManagedWidget("kanjilarge",
 						  labelWidgetClass,
 						  parent,
-			XtNwidth,FULLWIDTH - 100,
+			XtNwidth,SEARCHWIDTH - 100,
 			XtNheight,KANJIWIDTH,
 			XtNlabel,"",
 			XtNencoding,XawTextEncodingChar2b,
 			XtNfont,largekfont,
+			XtNright, XawChainRight,
+			XtNleft,  XawChainLeft,
 			NULL);
+	/* want to handle just middle button on this one */
+	XtAddEventHandler(searchkanjiW,
+			  ButtonPressMask,False,Handle_kanji_split,NULL);
+
+
 	matchbutton = XtVaCreateManagedWidget("matchkanjib",
 			commandWidgetClass,parent,
 			XtNfromHoriz,searchkanjiW,
 			XtNlabel, "match",
+			XtNright, XawChainRight,
+			XtNleft,  XawChainRight,
 			NULL);
 	
 	XtAddCallback(matchbutton, XtNcallback, Handle_matchkanji, NULL);
 
 	numbersform = XtVaCreateManagedWidget("numform",
 			formWidgetClass, parent,
+			XtNresizeToPreferred, True,
 			XtNfromVert,searchkanjiW,
+			XtNright, XawChainLeft,
+			XtNleft,  XawChainLeft,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
        			NULL);
 
 	makenumbers(numbersform);
@@ -571,39 +762,62 @@ void maketopsearchstuff(Widget parent){
  *
  * For kana, the "label" is actually a button that will bring up the
  * special kana input window. User can then point-and-click.
+ *
+ * Returns "lowest" (southernmost) widget, so status line can be
+ * "below" that.
  */
-void makesearchinputs(Widget parent)
+Widget makesearchinputs(Widget parent)
 {
-	Widget kanjisform;
+	Widget kanjisform, searchinputform;
+	Widget optsform, optslabel;
+
+
+	searchinputform=XtVaCreateManagedWidget("searchinputform",
+			formWidgetClass,parent,
+			XtNfromVert, searchwidgets[ENGLISH_W],
+			XtNvertDistance,10,
+			XtNresizeToPreferred, True,
+			XtNright, XawChainRight,
+			XtNleft,  XawChainLeft,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
+			NULL);
 
 	searchwidgets[SEARCH_ENG_L] =
-		XtVaCreateWidget("searchenglishlabel",
-			labelWidgetClass,
-			parent,
+		XtVaCreateManagedWidget("searchenglishlabel",
+			labelWidgetClass, searchinputform,
 			XtNjustify, XtJustifyRight,
 			XtNlabel, "English search",
-			XtNvertDistance, 20,
 			XtNborderWidth, 0,
-			XtNfromVert, searchwidgets[ENGLISH_W],
+			XtNleft, XawChainLeft,
+			XtNright, XawChainLeft,
 			NULL);
 						       
 	searchwidgets[SEARCH_ENG_W] =
-		XtVaCreateWidget("searchenglish",
+		XtVaCreateManagedWidget("searchenglish",
 			asciiTextWidgetClass,
-			parent, 
-			XtNfromVert, searchwidgets[ENGLISH_W],
+			searchinputform, 
 			XtNfromHoriz, searchwidgets[SEARCH_ENG_L],
-			XtNvertDistance, 20,
 			XtNfont, englishfont,
 			XtNeditType, XawtextEdit,
-			XtNwidth, FULLWIDTH - 150,
+			XtNleft, XawChainLeft,
+			XtNright, XawChainRight,
+			/*XtNstring,"                 ",*/
+			XtNwidth, 150,
 			NULL);
 
+	/************************************************************/
+
+
 	kanjisform=
 		XtVaCreateManagedWidget("searchkanjif",
-			formWidgetClass,parent,
+			formWidgetClass,searchinputform,
 			XtNfromVert, searchwidgets[SEARCH_ENG_L],
 			XtNvertDistance, 10,
+			XtNleft, XawChainLeft,
+			XtNright, XawChainLeft,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
 			NULL);
 
 	searchwidgets[SEARCH_KANJI_L] =
@@ -611,6 +825,7 @@ void makesearchinputs(Widget parent)
 			commandWidgetClass, kanjisform,
 			XtNjustify, XtJustifyRight,
 			XtNlabel, "Kanji search  ",
+			XtNright, XawChainRight,
 			NULL);
 
 	searchwidgets[SEARCH_SKIP_W] =
@@ -618,6 +833,8 @@ void makesearchinputs(Widget parent)
 			commandWidgetClass, kanjisform,
 			XtNlabel, "Kanji SKIP search",
 			XtNfromHoriz, searchwidgets[SEARCH_KANJI_L],
+			XtNleft, XawChainRight,
+			XtNright, XawChainRight,
 			NULL);
 
 	searchwidgets[SEARCH_STROKE_W] =
@@ -625,72 +842,72 @@ void makesearchinputs(Widget parent)
 			commandWidgetClass,kanjisform,
 			XtNjustify, XtJustifyRight,
 			XtNlabel, "Stroke count  ",
-			/*XtNborderWidth, 0,*/
 			XtNfromVert, searchwidgets[SEARCH_KANJI_L],
+			XtNright, XawChainRight,
 			NULL);
 
 
-	searchwidgets[SEARCH_KANJI_L2] =
-		XtVaCreateManagedWidget("searchkanjil2",
-			labelWidgetClass,
-			kanjisform,
-			XtNjustify, XtJustifyRight,
-			XtNlabel, "(or paste char ->)",
-			XtNborderWidth, 0,
-			XtNfromVert, searchwidgets[SEARCH_KANJI_L],
-			XtNfromHoriz, searchwidgets[SEARCH_STROKE_W],
-			NULL);
-
 	searchwidgets[SEARCH_KANJIPASTE] =
 		XtVaCreateManagedWidget("searchkanji",
-			asciiTextWidgetClass,
-			kanjisform, 
-			XtNwidth, 50,
-			XtNlabel, "",
+			labelWidgetClass,kanjisform,
+			XtNlabel," <paste kanji> ",
 			XtNfromVert, searchwidgets[SEARCH_KANJI_L],
-			XtNfromHoriz, searchwidgets[SEARCH_KANJI_L2],
-			XtNfont, smallkfont,
-			XtNencoding, XawTextEncodingChar2b,
-			XtNeditType, XawtextEdit,
-			XtNjustify, XtJustifyLeft,
-			XtNdisplayCaret, False,
+			XtNfromHoriz, searchwidgets[SEARCH_STROKE_W],
+			XtNleft, XawChainRight,
+			XtNright, XawChainRight,
 			NULL);
 
+	searchwidgets[SEARCH_RADICAL_W] =
+		XtVaCreateManagedWidget("searchradicals",
+			commandWidgetClass,kanjisform,
+			XtNlabel,"Radical search",
+			XtNfromVert, searchwidgets[SEARCH_STROKE_W],
+			XtNright, XawChainRight,
+			NULL);
 
 	/* and now the kana search input widgets. gets a bit messy here */
 
 	searchwidgets[KANA_FORM]=XtVaCreateManagedWidget("kanasearchgroup",
-			 formWidgetClass, parent,
-			 XtNfromVert, kanjisform,
-			 XtNvertDistance, 10,
-			 XtNborderWidth, 1,
-			 NULL);
-
-
-	searchwidgets[SEARCH_STARTTOG]=
-		XtVaCreateManagedWidget("atstart_toggle",
-			commandWidgetClass, parent,
+			formWidgetClass, searchinputform,
+			XtNresizeToPreferred, True,
 			XtNfromVert, kanjisform,
 			XtNvertDistance, 10,
-			XtNfromHoriz, searchwidgets[KANA_FORM],
-			XtNhorizDistance, 15,
-			XtNlabel, "[...",
+			XtNborderWidth, 1,
+			XtNright, XawChainRight,
+			XtNleft,  XawChainLeft,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
 			NULL);
-		
+
 
 	searchwidgets[SEARCH_KANA_L] =
 		XtVaCreateManagedWidget("searchkanalabel",
+			labelWidgetClass, searchwidgets[KANA_FORM],
+			XtNlabel, "Kana searches",
+			XtNborderWidth,0,
+			XtNright, XawChainLeft,
+			XtNleft, XawChainLeft,
+			NULL);
+
+	searchwidgets[SEARCH_KANA_B] =
+		XtVaCreateManagedWidget("searchkanabutton",
 			commandWidgetClass, searchwidgets[KANA_FORM],
-			XtNjustify, XtJustifyRight,
-			XtNlabel, "Kana search   ",
-			/*XtNborderWidth, 0,*/
+			XtNfromHoriz,searchwidgets[SEARCH_KANA_L],
+			XtNhorizDistance,20,
+			XtNlabel, "Popup Kana table",
+			XtNright, XawChainRight,
+			XtNleft, XawChainRight,
+			/* These dont work as I want */
+			/*
+			XtNresizable, False,
+			*/
 			NULL);
 
 
 	searchwidgets[SEARCH_KANA_W] =
 		XtVaCreateManagedWidget("searchkana",
 			labelWidgetClass, searchwidgets[KANA_FORM], 
-			XtNwidth, FULLWIDTH - 100,
+			XtNwidth, SEARCHWIDTH - 150,
 			XtNlabel,"\0",
 			/*XtNlabel, "!z",*/ /* 0x217a */
 			XtNfromVert, searchwidgets[SEARCH_KANA_L],
@@ -698,13 +915,54 @@ void makesearchinputs(Widget parent)
 			XtNencoding, XawTextEncodingChar2b,
 			XtNjustify, XtJustifyLeft,
 			XtNcursor, textcursor,
+			XtNright, XawChainRight,
+			XtNleft, XawChainLeft,
 			NULL);
 
 
-	/* This possibly now has the wrong value.(?) sigh. */
-	XtManageChildren(searchwidgets,SEARCH_ENG_W+1);
+	/************************************************************/
 
-	XtAddCallback(searchwidgets[SEARCH_KANA_L], 
+	optsform=XtVaCreateManagedWidget("searchoptsform",
+			formWidgetClass, parent,
+			XtNresizeToPreferred, True,
+			XtNfromVert, searchwidgets[ENGLISH_W],
+			XtNvertDistance,10,
+			XtNfromHoriz, searchinputform,
+			XtNborderWidth, 1,
+			XtNleft, XawChainRight,
+			XtNright, XawChainRight,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
+			NULL);
+
+
+	optslabel=XtVaCreateManagedWidget("searchoptslabel",
+			labelWidgetClass, optsform,
+			XtNlabel, " Match ",
+			XtNborderWidth, 0,
+			/*XtNjustify,XawCenter */ /* default */
+			NULL);
+			
+
+	searchwidgets[SEARCH_ACTIVEFILTER]=
+		XtVaCreateManagedWidget("filter_toggle",
+			commandWidgetClass, optsform,
+			XtNfromVert, optslabel,
+			XtNlabel, "active",
+			NULL);
+
+	searchwidgets[SEARCH_STARTTOG]=
+		XtVaCreateManagedWidget("atstart_toggle",
+			commandWidgetClass, optsform,
+			XtNfromVert, searchwidgets[SEARCH_ACTIVEFILTER],
+			XtNlabel, "initial",
+			NULL);
+		
+	/* This possibly now has the wrong value.(?) sigh. */
+	/*XtManageChildren(searchwidgets,SEARCH_ENG_W+1);*/
+
+
+	XtAddCallback(searchwidgets[SEARCH_KANA_B], 
 		      XtNcallback, Showinputkana, NULL);
 	XtAddCallback(searchwidgets[SEARCH_KANJI_L], 
 		      XtNcallback, Showinputkanji, NULL);
@@ -712,34 +970,42 @@ void makesearchinputs(Widget parent)
 		      XtNcallback, ShowSKIP, NULL);
 	XtAddCallback(searchwidgets[SEARCH_STROKE_W], 
 		      XtNcallback, Showstroke, NULL);
+	XtAddCallback(searchwidgets[SEARCH_RADICAL_W], 
+		      XtNcallback, ShowRadicalinput, NULL);
 
 	XtAddCallback(searchwidgets[SEARCH_STARTTOG], 
-		      XtNcallback, Handle_starttoggle, NULL);
+		      XtNcallback, ToggleMatchstart, NULL);
+	XtAddCallback(searchwidgets[SEARCH_ACTIVEFILTER], 
+		      XtNcallback, ToggleActivesearch, NULL);
 
 	XtAddCallback(searchnumbers[INUSEFILE], 
-		      XtNcallback, Handle_usetoggle, NULL);
+		      XtNcallback, ToggleUsefile, NULL);
 	XtAddCallback(searchnumbers[SHOWUSEFILE], 
-		      XtNcallback, Handle_showusefile, NULL);
+		      XtNcallback, ShowUsefile, NULL);
 
+	/*
+	XtAddCallback(searchwidgets[SEARCH_KANJIPASTE],
+		      XtNcallback, pasteCallback, NULL);
+	*/
+
+	/* want to handle any/all buttons */
+	XtAddEventHandler(searchwidgets[SEARCH_KANJIPASTE],
+			  ButtonPressMask,False,Handle_paste,NULL);
 
 
 	XtAddEventHandler(searchwidgets[SEARCH_KANA_W],
 			  KeyPressMask,
 			  False, Handle_romajikana,NULL);
 	
-	XtAddEventHandler(searchwidgets[SEARCH_KANJIPASTE],
-			  ButtonReleaseMask, False,
-			  Handle_searchPasted,NULL);
-			  
-
+	return searchinputform;
 }
 
 
 /* This is the only exported "make widgets" routine */
 void MakeSearchPopup()
 {
-	 Widget searchform;
-	 XtAccelerators Accel;
+	XtAccelerators Accel;
+	Widget searchform,lastsearchinput;
 
 	textcursor=XCreateFontCursor(display,XC_xterm);
 
@@ -750,7 +1016,8 @@ void MakeSearchPopup()
 		NULL);
 #else
 	search_popup = XtVaCreateWidget("kdrill_search",
-		shellWidgetClass,
+		/*shellWidgetClass,*/
+		topLevelShellWidgetClass,
 		toplevel,
 		NULL);
 	XtSetMappedWhenManaged(search_popup,False);
@@ -764,13 +1031,21 @@ void MakeSearchPopup()
 	searchwidgets[NFORM_W] = XtVaCreateManagedWidget("northform",
 			formWidgetClass,
 			searchform,
+			XtNresizeToPreferred, True,
 			XtNborderWidth,2,
+			XtNright, XawChainRight,
+			XtNleft,  XawChainLeft,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
 			NULL);
 
+	/* make everything in the top box: kanji, frequency, grade, etc*/
 	maketopsearchstuff(searchwidgets[NFORM_W]);
 
-	/* now make the long output display widgets */
-	searchwidgets[KANA_W] = XtVaCreateWidget("searchkana",
+	/* now make the wide output display widgets, below the main
+	 * kanji display, at the top.
+	 */
+	searchwidgets[KANA_W] = XtVaCreateManagedWidget("searchkana",
 						 labelWidgetClass,
 						 searchform,
 			XtNlabel,"                               ",
@@ -778,30 +1053,42 @@ void MakeSearchPopup()
 			XtNvertDistance,20,
 			XtNencoding,XawTextEncodingChar2b,
 			XtNfont,smallkfont,
-			XtNwidth,FULLWIDTH,
+			XtNwidth,SEARCHWIDTH,
+			XtNleft, XawChainLeft,
+			XtNright, XawChainRight,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
 			NULL);
 
-	searchwidgets[ENGLISH_W] = XtVaCreateWidget("searchenglish",
+	searchwidgets[ENGLISH_W] = XtVaCreateManagedWidget("searchenglish",
 						    labelWidgetClass,
 						    searchform, 
 			XtNlabel,"                                       ",
 			XtNfromVert,searchwidgets[KANA_W],
 			XtNfont,englishfont,
-			XtNwidth,FULLWIDTH,
+			XtNwidth,SEARCHWIDTH,
+			XtNleft, XawChainLeft,
+			XtNright, XawChainRight,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
 			NULL);
 
 	/* This makes other searchwidgets[] stuffs, below the static
 	 * display we just made, above.
 	 */
-	makesearchinputs(searchform);
+	lastsearchinput=makesearchinputs(searchform);
 
 	searchstatusline = XtVaCreateManagedWidget("searchstatus",
 			labelWidgetClass, searchform,
-		      XtNlabel,"Search popup",
-		      XtNwidth,FULLWIDTH,
-		      XtNborderWidth,2,
-		      XtNfromVert, searchwidgets[KANA_FORM],
-		      XtNresize,True,
+			XtNlabel,"Search popup",
+			XtNwidth,SEARCHWIDTH,
+			XtNborderWidth,2,
+			XtNfromVert, lastsearchinput,
+			XtNresize,True,
+			XtNleft, XawChainLeft,
+			XtNright, XawChainRight,
+			XtNtop,XawChainTop,
+			XtNbottom,XawChainTop,
 		      NULL);
 
 	Accel = XtParseAcceleratorTable(searchAccel);
