@@ -14,8 +14,16 @@ int		isapipe(FILE *);
 Boolean isMapped(Widget w);
 
 extern void ReverseButton(Widget);
-extern void UnreverseButton(Widget);
+extern void HighlightButton(Widget);
+extern void UnhighlightButton(Widget);
+extern void SetWhiteOnBlack(Widget);
+extern void SetBlackOnWhite(Widget);
 
 extern int GetWidgetNumberval(Widget);
 extern int GetWidgetHexval(Widget);
 extern int GetWidgetWidth(Widget);
+
+/* general-purpose widget operators.. */
+extern void SetWidgetNumberval(Widget,int);
+extern void SetWidgetHexval(Widget,int);
+
